package com.mbridge.sdk.demo;

import android.content.Context;
import android.text.TextUtils;

import com.mbridge.msdk.MBridgeConstans;
import com.mbridge.msdk.MBridgeSDK;
import com.mbridge.msdk.interstitialvideo.out.MBInterstitialVideoHandler;
import com.mbridge.msdk.out.MBRewardVideoHandler;
import com.mbridge.msdk.out.MBridgeSDKFactory;
import com.mbridge.msdk.out.SDKInitStatusListener;

import java.util.HashMap;
import java.util.Map;

public final class MBridgeSDKManager {
    private Context context;
    private volatile String appKey;
    private volatile String appID;
    private volatile MBridgeSDKInitializeListener sdkInitializeListener;
    private volatile Map<String, MBRewardVideoHandler> rewardVideoHandlerMap;
    private volatile Map<String, MBInterstitialVideoHandler> interstitialVideoHandlerMap;

    private static MBridgeSDKInitializeState currentState;
    private MBridgeSDK mBridgeSDK;

    private MBridgeSDKManager() {
        currentState = MBridgeSDKInitializeState.SDK_STATE_UN_INITIALIZE;
    }

    public static MBridgeSDKManager getInstance() {
        return ClassHolder.M_SDK_MANAGER;
    }

    public MBridgeSDKInitializeState getCurrentState() {
        return currentState;
    }

    public String getAppID() {
        return appID;
    }

    public String getAppKey() {
        return appKey;
    }

    public MBridgeSDK getMBridgeSDK() {
        return mBridgeSDK;
    }

    public synchronized void initialize(final Context context, final String appKey, final String appID, final boolean debug, final Map<String, String> map, final MBridgeSDKInitializeListener sdkInitializeListener) {

        if (currentState == MBridgeSDKInitializeState.SDK_STATE_INITIALIZING) {
            if (null != sdkInitializeListener) {
                sdkInitializeListener.onInitializeFailure("sdk is initializing");
            }
            return;
        }

        this.sdkInitializeListener = sdkInitializeListener;
        if (checkSDKInitializeEnvironment(context, appKey, appID)) {
            if (currentState == MBridgeSDKInitializeState.SDK_STATE_INITIALIZE_SUCCESS) {
                if (TextUtils.equals(this.appID, appID) && TextUtils.equals(this.appKey, appKey)) {
                    if (null != this.sdkInitializeListener) {
                        this.sdkInitializeListener.onInitializeSuccess(this.appKey, this.appID);
                    }
                    return;
                }
            }

            currentState = MBridgeSDKInitializeState.SDK_STATE_INITIALIZING;
            this.context = context;
            this.appKey = appKey;
            this.appID = appID;
            realSDKInitialize(debug, map, this.sdkInitializeListener);
        }
    }

    public synchronized void initialize(final Context context, final String appKey, final String appID, final boolean debug, final Map<String, String> map) {
        this.initialize(context, appKey, appID, debug, map, null);
    }


    public synchronized void initialize(final Context context, final String appKey, final String appID) {
        this.initialize(context, appKey, appID, false, null, null);
    }

    public synchronized void initialize(final Context context, final String appKey, final String appID, final boolean debug) {
        this.initialize(context, appKey, appID, debug, null, null);
    }

    public synchronized void initialize(final Context context, final String appKey, final String appID, final boolean debug, final MBridgeSDKInitializeListener sdkInitializeListener) {
        this.initialize(context, appKey, appID, debug, null, sdkInitializeListener);
    }

    public synchronized void initialize(final Context context, final String appKey, final String appID, final MBridgeSDKInitializeListener sdkInitializeListener) {
        this.initialize(context, appKey, appID, false, null, sdkInitializeListener);
    }

    private void realSDKInitialize(boolean debug, Map<String, String> map, MBridgeSDKInitializeListener sdkInitializeListener) {
        try {
            MBridgeConstans.DEBUG = debug;
            mBridgeSDK = MBridgeSDKFactory.getMBridgeSDK();
            Map<String, String> configurationMap = mBridgeSDK.getMBConfigurationMap(this.appID, this.appKey,"");
            if (null != map && !map.isEmpty()) {
                configurationMap.putAll(map);
            }
            mBridgeSDK.init(configurationMap, context, new DefaultSDKInitStatusListener(this.appKey, this.appID, this.sdkInitializeListener));
        } catch (Exception e) {
            currentState = MBridgeSDKInitializeState.SDK_STATE_INITIALIZE_FAILURE;
            if (null != this.sdkInitializeListener) {
                sdkInitializeListener.onInitializeFailure(e.getMessage());
            }
        }
    }

    private static final class ClassHolder {
        private static final MBridgeSDKManager M_SDK_MANAGER = new MBridgeSDKManager();
    }

    private static class DefaultSDKInitStatusListener implements SDKInitStatusListener {
        private String appKey;
        private String appID;
        private MBridgeSDKInitializeListener sdkInitializeListener;

        public DefaultSDKInitStatusListener(String appKey, String appID, MBridgeSDKInitializeListener sdkInitializeListener) {
            this.appKey = appKey;
            this.appID = appID;
            this.sdkInitializeListener = sdkInitializeListener;
        }

        @Override
        public void onInitSuccess() {
            currentState = MBridgeSDKInitializeState.SDK_STATE_INITIALIZE_SUCCESS;
            if (null != sdkInitializeListener) {
                sdkInitializeListener.onInitializeSuccess(this.appKey, this.appID);
            }
        }

        @Override
        public void onInitFail(String errorMsg) {
            currentState = MBridgeSDKInitializeState.SDK_STATE_INITIALIZE_FAILURE;
            if (null != sdkInitializeListener) {
                sdkInitializeListener.onInitializeFailure("sdk initialize failed： an exception occurs");
            }
        }
    }

    private boolean checkSDKInitializeEnvironment(final Context context, final String appKey, final String appID) {
        boolean environmentAvailable = true;
        String errorMessage = "";
        if (null == context) {
            environmentAvailable = false;
            errorMessage = "context must not null";
        }

        if (TextUtils.isEmpty(appKey) || TextUtils.isEmpty(appID)) {
            environmentAvailable = false;
            errorMessage = TextUtils.isEmpty(errorMessage) ? "appKey or appID must not null" : errorMessage + " & appKey or appID must not null";
        }

        if (!environmentAvailable && !TextUtils.isEmpty(errorMessage)) {
            if (null != sdkInitializeListener) {
                currentState = MBridgeSDKInitializeState.SDK_STATE_INITIALIZE_FAILURE;
                sdkInitializeListener.onInitializeFailure(errorMessage);
            }
        }
        return environmentAvailable;
    }

    public interface MBridgeSDKInitializeListener {
        void onInitializeSuccess(String appKey, String appID);
        void onInitializeFailure(String message);
    }

    public enum MBridgeSDKInitializeState {
        SDK_STATE_UN_INITIALIZE,
        SDK_STATE_INITIALIZING,
        SDK_STATE_INITIALIZE_SUCCESS,
        SDK_STATE_INITIALIZE_FAILURE
    }

    /**
     * create MBRewardVideoHandler
     *
     * @param context
     * @param placementID
     * @param unitID
     * @return
     */
    public synchronized MBRewardVideoHandler createRewardVideoHandler(Context context, String placementID, String unitID) {

        if (rewardVideoHandlerMap == null) {
            rewardVideoHandlerMap = new HashMap<>();
        }

        MBRewardVideoHandler rewardVideoHandler = null;
        if (!TextUtils.isEmpty(unitID)) {
            if (rewardVideoHandlerMap.containsKey(unitID)) {
                rewardVideoHandler = rewardVideoHandlerMap.get(unitID);
            } else {
                rewardVideoHandler = new MBRewardVideoHandler(context, placementID, unitID);
                rewardVideoHandlerMap.put(unitID, rewardVideoHandler);
            }
        }
        return rewardVideoHandler;
    }

    /**
     * create MBRewardVideoHandler
     *
     * @param placementID
     * @param unitID
     * @return
     */
    public synchronized MBRewardVideoHandler createRewardVideoHandler(String placementID, String unitID) {
        return createRewardVideoHandler(null, placementID, unitID);
    }

    /**
     * release MBRewardVideoHandler
     *
     * @param unitIDs
     */
    public synchronized void releaseRewardVideoHandler(String... unitIDs) {
        if (rewardVideoHandlerMap != null && unitIDs != null && unitIDs.length > 0) {
            for (String unitID : unitIDs) {
                if (!TextUtils.isEmpty(unitID)) {
                    rewardVideoHandlerMap.remove(unitID);
                }
            }
        }
    }

    /**
     * create MBInterstitialVideoHandler
     *
     * @param context
     * @param placementID
     * @param unitID
     * @return
     */
    public synchronized MBInterstitialVideoHandler createInterstitialVideoHandler(Context context, String placementID, String unitID) {
        if (interstitialVideoHandlerMap == null) {
            interstitialVideoHandlerMap = new HashMap<String, MBInterstitialVideoHandler>();
        }

        MBInterstitialVideoHandler interstitialVideoHandler = null;
        if (interstitialVideoHandlerMap.containsKey(unitID)) {
            interstitialVideoHandler = interstitialVideoHandlerMap.get(unitID);
        } else {
            interstitialVideoHandler = new MBInterstitialVideoHandler(context, placementID, unitID);
            interstitialVideoHandlerMap.put(unitID, interstitialVideoHandler);
        }
        return interstitialVideoHandler;
    }

    /**
     * create MBInterstitialVideoHandler
     *
     * @param placementID
     * @param unitID
     * @return
     */
    public synchronized MBInterstitialVideoHandler createInterstitialVideoHandler(String placementID, String unitID) {
        return createInterstitialVideoHandler(null, placementID, unitID);
    }

    /**
     * release MBInterstitialVideoHandler
     *
     * @param unitIDs
     */
    public synchronized void releaseInterstitialVideoHandler(String... unitIDs) {
        if (interstitialVideoHandlerMap != null && unitIDs != null && unitIDs.length > 0) {
            for (String unitID : unitIDs) {
                if (!TextUtils.isEmpty(unitID)) {
                    interstitialVideoHandlerMap.remove(unitID);
                }
            }
        }
    }
}
