package com.squareup.leakcanary;

import static android.text.format.Formatter.formatShortFileSize;
import static com.squareup.leakcanary.BuildConfig.GIT_SHA;
import static com.squareup.leakcanary.BuildConfig.LIBRARY_VERSION;
import static com.squareup.leakcanary.internal.LeakCanaryInternals.isInServiceProcess;

import android.app.Application;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.util.Log;

import androidx.annotation.NonNull;

import com.squareup.leakcanary.internal.DisplayLeakActivity;
import com.squareup.leakcanary.internal.HeapAnalyzerService;
import com.squareup.leakcanary.internal.LeakCanaryInternals;

public final class LeakCanaryNoDisplay {
    public static @NonNull
    RefWatcher install(@NonNull Application application) {
        return refWatcher(application).listenerServiceClass(LeakNotifyService.class)
                .excludedRefs(AndroidExcludedRefs.createAppDefaults().build())
                .buildAndInstall();
    }

    /**
     * Returns the {@link RefWatcher} installed via
     * {@link LeakRefWatcherBuilderNoToast#buildAndInstall()}, and {@link RefWatcher#DISABLED} is no
     * {@link RefWatcher} has been installed.
     */
    public static @NonNull
    RefWatcher installedRefWatcher() {
        RefWatcher refWatcher = LeakCanaryInternals.installedRefWatcher;
        if (refWatcher == null) {
            return RefWatcher.DISABLED;
        }
        return refWatcher;
    }

    public static @NonNull
    LeakRefWatcherBuilderNoToast refWatcher(@NonNull Context context) {
        return new LeakRefWatcherBuilderNoToast(context);
    }

    /**
     * Blocking inter process call that enables the {@link DisplayLeakActivity}. When you first
     * install the app, {@link DisplayLeakActivity} is disabled by default and will only be enabled
     * once a potential leak has been found and the analysis starts. You can call this method to
     * enable {@link DisplayLeakActivity} before any potential leak has been detected.
     */
    public static void enableDisplayLeakActivity(@NonNull Context context) {
        LeakCanaryInternals.setEnabledBlocking(context, DisplayLeakActivity.class, false);
    }

    /**
     * @deprecated Use {@link #setLeakDirectoryProvider(LeakDirectoryProvider)} instead.
     */
    @Deprecated
    public static void setDisplayLeakActivityDirectoryProvider(
            @NonNull LeakDirectoryProvider leakDirectoryProvider) {
        setLeakDirectoryProvider(leakDirectoryProvider);
    }

    /**
     * Used to customize the location for the storage of heap dumps. The default implementation is
     * {@link DefaultLeakDirectoryProvider}.
     *
     * @throws IllegalStateException if a LeakDirectoryProvider has already been set, including
     *                               if the default has been automatically set when installing the ref watcher.
     */
    public static void setLeakDirectoryProvider(
            @NonNull LeakDirectoryProvider leakDirectoryProvider) {
        LeakCanaryInternals.setLeakDirectoryProvider(leakDirectoryProvider);
    }

    /**
     * Returns a string representation of the result of a heap analysis.
     */
    public static @NonNull
    String leakInfo(@NonNull Context context,
                    @NonNull HeapDump heapDump,
                    @NonNull AnalysisResult result,
                    boolean detailed) {
        PackageManager packageManager = context.getPackageManager();
        String packageName = context.getPackageName();
        PackageInfo packageInfo;
        try {
            packageInfo = packageManager.getPackageInfo(packageName, 0);
        } catch (PackageManager.NameNotFoundException e) {
            throw new RuntimeException(e);
        }
        String versionName = packageInfo.versionName;
        int versionCode = packageInfo.versionCode;
        String info = "In " + packageName + ":" + versionName + ":" + versionCode + ".\n";
        String detailedString = "";
        if (result.leakFound) {
            if (result.excludedLeak) {
                info += "* EXCLUDED LEAK.\n";
            }
            info += "* " + result.className;
            if (!heapDump.referenceName.equals("")) {
                info += " (" + heapDump.referenceName + ")";
            }
            info += " has leaked:\n" + result.leakTrace.toString() + "\n";
            if (result.retainedHeapSize != AnalysisResult.RETAINED_HEAP_SKIPPED) {
                info += "* Retaining: " + formatShortFileSize(context, result.retainedHeapSize) + ".\n";
            }
            if (detailed) {
                detailedString = "\n* Details:\n" + result.leakTrace.toDetailedString();
            }
        } else if (result.failure != null) {
            // We duplicate the library version & Sha information because bug reports often only contain
            // the stacktrace.
            info += "* FAILURE in " + LIBRARY_VERSION + " " + GIT_SHA + ":" + Log.getStackTraceString(
                    result.failure) + "\n";
        } else {
            info += "* NO LEAK FOUND.\n\n";
        }
        if (detailed) {
            detailedString += "* Excluded Refs:\n" + heapDump.excludedRefs;
        }

        info += "* Reference Key: "
                + heapDump.referenceKey
                + "\n"
                + "* Device: "
                + Build.MANUFACTURER
                + " "
                + Build.BRAND
                + " "
                + Build.MODEL
                + " "
                + Build.PRODUCT
                + "\n"
                + "* Android Version: "
                + Build.VERSION.RELEASE
                + " API: "
                + Build.VERSION.SDK_INT
                + " LeakCanary: "
                + LIBRARY_VERSION
                + " "
                + GIT_SHA
                + "\n"
                + "* Durations: watch="
                + heapDump.watchDurationMs
                + "ms, gc="
                + heapDump.gcDurationMs
                + "ms, heap dump="
                + heapDump.heapDumpDurationMs
                + "ms, analysis="
                + result.analysisDurationMs
                + "ms"
                + "\n"
                + detailedString;

        return info;
    }

    /**
     * Whether the current process is the process running the {@link HeapAnalyzerService}, which is
     * a different process than the normal app process.
     */
    public static boolean isInAnalyzerProcess(@NonNull Context context) {
        Boolean isInAnalyzerProcess = LeakCanaryInternals.isInAnalyzerProcess;
        // This only needs to be computed once per process.
        if (isInAnalyzerProcess == null) {
            isInAnalyzerProcess = isInServiceProcess(context, HeapAnalyzerService.class);
            LeakCanaryInternals.isInAnalyzerProcess = isInAnalyzerProcess;
        }
        return isInAnalyzerProcess;
    }

    private LeakCanaryNoDisplay() {
        throw new AssertionError();
    }

    public interface LeakCanaryCallBack {
        void onAnalysisResult(String result);
    }

    private static LeakCanaryCallBack sLeakCanaryCallBack;

    public static LeakCanaryCallBack getLeakCanaryCallBack() {
        return sLeakCanaryCallBack;
    }
}
